@@ -1,5 +1,5 @@
 # This file is part of IFM (Interactive Fiction Mapper), copyright (C)
-# Glenn Hutchings 1997-2008.
+# Glenn Hutchings 1997-2015.
 
 # IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
 # are welcome to redistribute it under certain conditions; see the file
@@ -53,7 +53,7 @@ foreach $opt (keys %ifmopts) {
 $file = shift(@ARGV) or die "$0: error: no IFM file specified\n";
 
 # Get no. of map sections to output.
-@data = `$ifm -show maps $file`;
+@data = `$ifm -m -f text $file`;
 shift(@data);
 $count = @data;
 die "$0: error: can't find any map sections\n" if $count <= 0;
