@@ -1,5 +1,5 @@
 # This file is part of IFM (Interactive Fiction Mapper), copyright (C)
-# Glenn Hutchings 1997-2008.
+# Glenn Hutchings 1997-2015.
 
 # IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
 # are welcome to redistribute it under certain conditions; see the file
@@ -85,7 +85,7 @@ if ($opts{m}) {
     system "ifm2dev -o $template -- -w $file";
 
     # Get dimensions of each one.
-    @data = `$ifm -show maps $file`;
+    @data = `$ifm -m -f text $file`;
     shift(@data);
 
     # Write each map section.
