@@ -1,13 +1,13 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
  * COPYING for details.
  */
 
-/* Fig output driver */
+/* Fig output format */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -22,7 +22,7 @@
 #include <vars.h>
 #include <fig.h>
 
-#include "ifm-driver.h"
+#include "ifm-format.h"
 #include "ifm-fig.h"
 #include "ifm-main.h"
 #include "ifm-map.h"
@@ -66,6 +66,7 @@ mapfuncs fig_mapfuncs = {
 /* Internal functions */
 static void set_colour(vhash *object, char *colour);
 static void set_fillcolour(vhash *object, char *colour);
+static void output_handler(char *line);
 
 /* Map functions */
 void
@@ -76,6 +77,10 @@ fig_map_start(void)
     vhash *sect, *box;
     viter iter;
 
+    /* Initialize */
+    if (fig != NULL)
+        v_destroy(fig);
+
     /* Allow title space for sections with titles */
     v_iterate(sects, iter) {
         sect = vl_iter_pval(iter);
@@ -433,8 +438,8 @@ fig_map_link(vhash *link)
 void
 fig_map_finish(void)
 {
-    /* Write figure */
-    fig_write_figure(fig, stdout);
+    fig_set_handler(output_handler);
+    fig_write_figure(fig);
 }
 
 /* Set the colour of an object */
@@ -456,3 +461,9 @@ set_fillcolour(vhash *object, char *colour)
     if (sscanf(colour, "%f %f %f", &r, &g, &b) == 3)
         fig_set_fillcolour(object, r, g, b);
 }
+
+static void
+output_handler(char *line)
+{
+    output(line);
+}
