@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -18,7 +18,7 @@
 #include <string.h>
 #include <vars.h>
 
-#include "ifm-driver.h"
+#include "ifm-format.h"
 #include "ifm-main.h"
 #include "ifm-map.h"
 #include "ifm-util.h"
@@ -46,7 +46,9 @@ vhash *tasktags = NULL;         /* Tag -> task mapping */
 
 vhash *startroom = NULL;        /* Starting room */
 
+vlist *tasklist = NULL;         /* Task step list */
 vlist *taskorder = NULL;        /* Ordered task list */
+
 vlist *sectnames = NULL;	/* List of section names */
 
 /* Internal stuff */
@@ -62,7 +64,14 @@ static void resolve_tag_list(char *type, vlist *list, vhash *table);
 void
 init_map(void)
 {
+    if (map != NULL)
+        v_destroy(map);
+
     map = vh_create();
+    vh_sstore(map, "TITLE", "Interactive Fiction map");
+
+    startroom = NULL;
+
     rooms = vl_create();
     links = vl_create();
     items = vl_create();
@@ -78,19 +87,25 @@ init_map(void)
     jointags = vh_create();
     tasktags = vh_create();
 
+    tasklist = vl_create();
     taskorder = vl_create();
 
     vh_pstore(map, "ROOMS", rooms);
     vh_pstore(map, "LINKS", links);
     vh_pstore(map, "ITEMS", items);
     vh_pstore(map, "JOINS", joins);
-    vh_pstore(map, "TASKS", taskorder);
     vh_pstore(map, "SECTS", sects);
 
+    vh_pstore(map, "TASKS", taskorder);
+    vh_pstore(map, "TSTEP", tasklist);
+    vh_pstore(map, "TDATA", tasks);
+
     vh_pstore(map, "RPOS", rpos);
 
     vh_pstore(map, "ROOMTAGS", roomtags);
     vh_pstore(map, "ITEMTAGS", itemtags);
+    vh_pstore(map, "LINKTAGS", linktags);
+    vh_pstore(map, "JOINTAGS", jointags);
     vh_pstore(map, "TASKTAGS", tasktags);
 }
 
@@ -100,7 +115,7 @@ put_room_at(vhash * room, int sect, int x, int y)
 {
     V_BUF_DECL;
 
-    V_BUF_SET3("%d,%d,%d", sect, x, y);
+    V_BUF_SETF("%d,%d,%d", sect, x, y);
     vh_pstore(rpos, V_BUF_VAL, room);
     vh_istore(room, "X", x);
     vh_istore(room, "Y", y);
@@ -237,7 +252,7 @@ room_at(int sect, int x, int y)
 {
     V_BUF_DECL;
 
-    V_BUF_SET3("%d,%d,%d", sect, x, y);
+    V_BUF_SETF("%d,%d,%d", sect, x, y);
     return vh_pget(rpos, V_BUF_VAL);
 }
 
@@ -250,10 +265,10 @@ room_exit(vhash *room, int xoff, int yoff, int flag)
     int num;
 
     if (xoff == 0 && yoff == 0)
-        fatal("internal: invalid direction offset");
+        err("internal: invalid direction offset");
 
     flags = vh_add_hash(room, "EXIT");
-    V_BUF_SET2("%d,%d", xoff, yoff);
+    V_BUF_SETF("%d,%d", xoff, yoff);
     num = vh_iget(flags, V_BUF_VAL);
     num = V_MAX(num, 0);
 
@@ -322,7 +337,7 @@ setup_exits(void)
 void
 setup_links(void)
 {
-    int x, y, xt, yt, xoff, yoff, dir, num, count, ndirs;
+    int x, y, xt, yt, xoff = 0, yoff = 0, dir, num, count, ndirs;
     vhash *link, *from, *to, *other, *sect;
     vlist *dirs, *xpos, *ypos, *list;
     char *fname, *tname;
@@ -547,20 +562,23 @@ setup_rooms(void)
 void
 setup_sections(void)
 {
-    int minx, miny, maxx, maxy, x, y, first, pos;
+    int minx = 0, miny = 0, maxx = 0, maxy = 0, x, y, first, pos;
     vhash *sect, *room, *link;
     vlist *list, *xpos, *ypos;
     viter i, j, k;
     char *title;
+    V_BUF_DECL;
 
     v_iterate(sects, i) {
         sect = vl_iter_pval(i);
 
         /* Set title */
-        if (sectnames != NULL && vl_length(sectnames) > 0) {
+        if (sectnames != NULL && vl_length(sectnames) > 0)
             title = vl_sshift(sectnames);
-            vh_sstore(sect, "TITLE", title);
-        }
+        else
+            title = V_BUF_SETF("Map section %d", v_iter_count(i) + 1);
+
+        vh_sstore(sect, "TITLE", title);
 
         /* Find width and length of section */
         first = 1;
