@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -16,7 +16,7 @@
 extern vhash *map, *startroom;
 extern vlist *rooms, *links, *joins, *items, *tasks, *sects;
 extern vhash *roomtags, *itemtags, *tasktags, *linktags, *jointags;
-extern vlist *taskorder, *sectnames;
+extern vlist *tasklist, *taskorder, *sectnames;
 
 /* Advertised functions */
 extern void init_map(void);
