@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -23,6 +23,7 @@
 #include "ifm-path.h"
 #include "ifm-task.h"
 #include "ifm-util.h"
+#include "ifm-vars.h"
 
 #define NODE(room)           vh_sgetref(room, "NODE")
 
@@ -56,6 +57,24 @@ static int sort_tasks(vscalar **v1, vscalar **v2);
 static int use_link(char *fnode, char *tnode, vscalar *s);
 static int use_node(char *node, vscalar *s, double dist);
 
+/* Initialize paths */
+void
+init_paths(void)
+{
+    if (graph != NULL)
+        vg_destroy(graph);
+
+    graph = NULL;
+
+    path_room = NULL;
+    path_task = NULL;
+    path_modify = 0;
+
+#ifdef SHOW_VISIT
+    start_room = NULL;
+#endif
+}
+
 /* Connect rooms as a directed graph */
 void
 connect_rooms(void)
@@ -63,7 +82,7 @@ connect_rooms(void)
     int oneway, len, goflag, dir, num = 0, uselen = 0;
     vhash *room, *link, *join, *reach, *from, *to;
     vlist *cmdfrom, *cmdto, *list;
-    char *cmd, id[10];
+    char *cmd, id[20];
     viter iter;
 
     solver_msg(0, "Connecting rooms...");
@@ -127,13 +146,13 @@ connect_rooms(void)
         if (TASK_VERBOSE) {
             indent(1);
             list = vh_pget(reach, "CMD");
-            printf("link '%s' to '%s' (%s)",
+            output("link '%s' to '%s' (%s)",
                    vh_sgetref(from, "DESC"),
                    vh_sgetref(to, "DESC"),
                    vl_join(list, ". "));
             if (len > 1)
-                printf(" (dist %d)", len);
-            printf("\n");
+                output(" (dist %d)", len);
+            output("\n");
         }
 
         /* To -> from if not one-way */
@@ -169,13 +188,13 @@ connect_rooms(void)
             if (TASK_VERBOSE) {
                 indent(1);
                 list = vh_pget(reach, "CMD");
-                printf("link '%s' to '%s' (%s)",
+                output("link '%s' to '%s' (%s)",
                        vh_sgetref(to, "DESC"),
                        vh_sgetref(from, "DESC"),
                        vl_join(list, ". "));
                 if (len > 1)
-                    printf(" (dist %d)", len);
-                printf("\n");
+                    output(" (dist %d)", len);
+                output("\n");
             }
         }
     }
@@ -223,13 +242,13 @@ connect_rooms(void)
         if (TASK_VERBOSE) {
             indent(1);
             list = vh_pget(reach, "CMD");
-            printf("join '%s' to '%s' (%s)",
+            output("join '%s' to '%s' (%s)",
                    vh_sgetref(from, "DESC"),
                    vh_sgetref(to, "DESC"),
                    vl_join(list, ". "));
             if (len > 1)
-                printf(" (dist %d)", len);
-            printf("\n");
+                output(" (dist %d)", len);
+            output("\n");
         }
 
         /* To -> from if not one-way */
@@ -264,13 +283,13 @@ connect_rooms(void)
             if (TASK_VERBOSE) {
                 indent(1);
                 list = vh_pget(reach, "CMD");
-                printf("join '%s' to '%s' (%s)",
+                output("join '%s' to '%s' (%s)",
                        vh_sgetref(to, "DESC"),
                        vh_sgetref(from, "DESC"),
                        vl_join(list, ". "));
                 if (len > 1)
-                    printf(" (dist %d)", len);
-                printf("\n");
+                    output(" (dist %d)", len);
+                output("\n");
             }
         }
     }
@@ -300,14 +319,14 @@ find_path(vhash *step, vhash *from, vhash *to)
 
     if (TASK_VERBOSE) {
         indent(3);
-        printf("find path: '%s' to '%s'",
+        output("find path: '%s' to '%s'",
                vh_sgetref(from, "DESC"),
                vh_sgetref(to, "DESC"));
     }
 
     if (step != NULL && vh_iget(step, "BLOCK")) {
         if (TASK_VERBOSE)
-            printf("\n");
+            output("\n");
 
         path_task = step;
         if ((path = vh_pget(step, "PATH")) != NULL)
@@ -325,15 +344,15 @@ find_path(vhash *step, vhash *from, vhash *to)
         vg_use_cache(graph, 1);
 
         if (TASK_VERBOSE && path_room != from)
-            printf("\n");
+            output("\n");
 
         len = PATH_LENGTH(from, to);
 
         if (TASK_VERBOSE && path_room == from) {
             if (len < 0)
-                printf(" (cached: no path)\n");
+                output(" (cached: no path)\n");
             else
-                printf(" (cached: dist %d)\n", len);
+                output(" (cached: dist %d)\n", len);
         }
     }
 
@@ -439,9 +458,9 @@ init_path(vhash *room)
 
             if (TASK_VERBOSE) {
                 indent(2);
-                printf("update path: %s\n", vh_sgetref(step, "DESC"));
+                output("update path: %s\n", vh_sgetref(step, "DESC"));
                 indent(2);
-                printf("possible block: %s may need dropping\n",
+                output("possible block: %s may need dropping\n",
                        vh_sgetref(item, "DESC"));
             }
 
@@ -493,10 +512,10 @@ init_path(vhash *room)
                 continue;
 
             indent(3);
-            printf("dist %d: %s", len, vh_sgetref(step, "DESC"));
+            output("dist %d: %s", len, vh_sgetref(step, "DESC"));
             if (len > 0 && (room = vh_pget(step, "ROOM")) != NULL)
-                printf(" (%s)", vh_sgetref(room, "DESC"));
-            printf("\n");
+                output(" (%s)", vh_sgetref(room, "DESC"));
+            output("\n");
         }
     }
 }
@@ -633,7 +652,7 @@ use_link(char *fnode, char *tnode, vscalar *s)
                     if (TASK_VERBOSE) {
                         indent(4 - vg_caching());
                         room = vg_node_pget(graph, tnode);
-                        printf("blocked link: %s (must leave %s)\n",
+                        output("blocked link: %s (must leave %s)\n",
                                vh_sgetref(room, "DESC"),
                                vh_sgetref(item, "DESC"));
                     }
@@ -655,7 +674,7 @@ use_link(char *fnode, char *tnode, vscalar *s)
                     if (TASK_VERBOSE) {
                         indent(4 - vg_caching());
                         room = vg_node_pget(graph, tnode);
-                        printf("blocked link: %s (need %s)\n",
+                        output("blocked link: %s (need %s)\n",
                                vh_sgetref(room, "DESC"),
                                vh_sgetref(item, "DESC"));
                     }
@@ -678,7 +697,7 @@ use_link(char *fnode, char *tnode, vscalar *s)
                     if (TASK_VERBOSE) {
                         indent(4 - vg_caching());
                         room = vg_node_pget(graph, tnode);
-                        printf("blocked link: %s (done '%s')\n",
+                        output("blocked link: %s (done '%s')\n",
                                vh_sgetref(room, "DESC"),
                                vh_sgetref(task, "DESC"));
                     }
@@ -700,7 +719,7 @@ use_link(char *fnode, char *tnode, vscalar *s)
                     if (TASK_VERBOSE) {
                         indent(4 - vg_caching());
                         room = vg_node_pget(graph, tnode);
-                        printf("blocked link: %s (not done '%s')\n",
+                        output("blocked link: %s (not done '%s')\n",
                                vh_sgetref(room, "DESC"),
                                vh_sgetref(task, "DESC"));
                     }
@@ -740,7 +759,7 @@ use_node(char *node, vscalar *s, double dist)
             if (block != NULL && block == path_task) {
                 if (TASK_VERBOSE) {
                     indent(4 - vg_caching());
-                    printf("blocked room: %s (must leave %s)\n",
+                    output("blocked room: %s (must leave %s)\n",
                            vh_sgetref(room, "DESC"),
                            vh_sgetref(item, "DESC"));
                 }
@@ -757,7 +776,7 @@ use_node(char *node, vscalar *s, double dist)
             if (!vh_iget(item, "TAKEN")) {
                 if (TASK_VERBOSE) {
                     indent(4 - vg_caching());
-                    printf("blocked room: %s (need %s)\n",
+                    output("blocked room: %s (need %s)\n",
                            vh_sgetref(room, "DESC"),
                            vh_sgetref(item, "DESC"));
                 }
@@ -775,7 +794,7 @@ use_node(char *node, vscalar *s, double dist)
             if (vh_iget(tstep, "DONE")) {
                 if (TASK_VERBOSE) {
                     indent(4 - vg_caching());
-                    printf("blocked room: %s (done '%s')\n",
+                    output("blocked room: %s (done '%s')\n",
                            vh_sgetref(room, "DESC"),
                            vh_sgetref(task, "DESC"));
                 }
@@ -792,7 +811,7 @@ use_node(char *node, vscalar *s, double dist)
             if (!vh_iget(tstep, "DONE")) {
                 if (TASK_VERBOSE) {
                     indent(4 - vg_caching());
-                    printf("blocked room: %s (not done '%s')\n",
+                    output("blocked room: %s (not done '%s')\n",
                            vh_sgetref(room, "DESC"),
                            vh_sgetref(task, "DESC"));
                 }
@@ -805,7 +824,7 @@ use_node(char *node, vscalar *s, double dist)
 #ifdef SHOW_VISIT
     if (TASK_VERBOSE && room != start_room) {
         indent(4 - vg_caching());
-        printf("visit: %s (dist %g)\n",
+        output("visit: %s (dist %g)\n",
                vh_sgetref(room, "DESC"), dist);
     }
 #endif
