@@ -1,13 +1,13 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
  * COPYING for details.
  */
 
-/* PostScript output driver */
+/* PostScript output format */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -21,7 +21,7 @@
 #include <math.h>
 #include <vars.h>
 
-#include "ifm-driver.h"
+#include "ifm-format.h"
 #include "ifm-main.h"
 #include "ifm-map.h"
 #include "ifm-util.h"
@@ -30,31 +30,31 @@
 
 #define PRINT_COLOUR(name) \
         if (var_changed(#name)) \
-                printf("/%s [%s] def\n", #name, var_colour(#name))
+                output("/%s [%s] def\n", #name, var_colour(#name))
 
 #define PRINT_FONT(name) \
         if (var_changed(#name)) \
-                printf("/%s /%s def\n", #name, var_string(#name))
+                output("/%s /%s def\n", #name, var_string(#name))
 
 #define PRINT_FONTSIZE(name) \
         if (var_changed(#name)) \
-                printf("/%s %g def\n", #name, var_real(#name) * font_scale)
+                output("/%s %g def\n", #name, var_real(#name) * font_scale)
 
 #define PRINT_INT(name) \
         if (var_changed(#name)) \
-                printf("/%s %d def\n", #name, var_int(#name))
+                output("/%s %d def\n", #name, var_int(#name))
 
 #define PRINT_REAL(name) \
         if (var_changed(#name)) \
-                printf("/%s %g def\n", #name, var_real(#name))
+                output("/%s %g def\n", #name, var_real(#name))
 
 #define PRINT_STRING(name) \
         if (var_changed(#name)) \
-                printf("/%s %s def\n", #name, ps_string(var_string(#name)))
+                output("/%s %s def\n", #name, ps_string(var_string(#name)))
 
 #define PRINT_BOOL(name) \
         if (var_changed(#name)) \
-                printf("/%s %s def\n", #name, var_int(#name) ? "true" : "false")
+                output("/%s %s def\n", #name, var_int(#name) ? "true" : "false")
 
 /* Map function list */
 mapfuncs ps_mapfuncs = {
@@ -84,16 +84,22 @@ static char *ps_string(char *str);
 void
 ps_map_start(void)
 {
-    int ylen, c, num_pages, width, height;
-    char *title, *prolog, *file;
+    int ylen, num_pages, width, height;
+    char *title, *prolog, *file, *line;
+    FILE *fp = NULL;
     vhash *sect;
     viter iter;
-    FILE *fp;
+    V_BUF_DECL;
+
+    /* Initialise */
+    ps_rotate = 0;
+    ps_rotflag = 0;
+    ps_pagenum = 0;
 
     /* Locate prolog file */
     file = var_string("prolog_file");
     if ((prolog = find_file(file)) == NULL)
-        fatal("can't find PostScript prolog '%s'", file);
+        err("can't find PostScript prolog '%s'", file);
 
     /* Allow title space for sections with titles */
     v_iterate(sects, iter) {
@@ -120,32 +126,29 @@ ps_map_start(void)
     num_pages = pack_sections(width, height);
 
     /* Print header */
-    if (vh_exists(map, "TITLE"))
-        title = vh_sgetref(map, "TITLE");
-    else
-        title = "Interactive Fiction map";
+    title = vh_sgetref(map, "TITLE");
 
-    printf("%%!PS-Adobe-2.0\n");
-    put_string("%%%%Title: %s\n", title);
-    printf("%%%%Creator: IFM v%s\n", VERSION);
-    printf("%%%%Pages: %d\n", num_pages);
-    printf("%%%%EndComments\n\n");
+    output("%%!PS-Adobe-2.0\n");
+    output("%%%%Title: %s\n", title);
+    output("%%%%Creator: IFM v%s\n", VERSION);
+    output("%%%%Pages: %d\n", num_pages);
+    output("%%%%EndComments\n\n");
 
     /* Print PostScript prolog */
-    if ((fp = fopen(prolog, "r")) == NULL)
-        fatal("can't open '%s'", prolog);
-
-    while ((c = fgetc(fp)) != EOF)
-        putchar(c);
-
-    fclose(fp);
+    if (prolog != NULL && (fp = fopen(prolog, "r")) != NULL) {
+        while ((line = V_BUF_FGETS(fp)) != NULL)
+            output("%s", line);
+        fclose(fp);
+    } else {
+        err("can't read '%s'", prolog);
+    }
 
     /* Page variables */
-    printf("/page_margin %g cm def\n", var_real("page_margin"));
-    printf("/page_width %g cm def\n", page_width);
-    printf("/page_height %g cm def\n", page_height);
-    printf("/map_width %d def\n", width);
-    printf("/map_height %d def\n", height);
+    output("/page_margin %g cm def\n", var_real("page_margin"));
+    output("/page_width %g cm def\n", page_width);
+    output("/page_height %g cm def\n", page_height);
+    output("/map_width %d def\n", width);
+    output("/map_height %d def\n", height);
 
     PRINT_BOOL(show_page_border);
     PRINT_COLOUR(page_border_colour);
@@ -154,12 +157,12 @@ ps_map_start(void)
     /* Title variables */
     if (title != NULL) {
         PRINT_BOOL(show_page_title);
-        put_string("/titlestring %s def\n", ps_string(title));
+        output("/titlestring %s def\n", ps_string(title));
         PRINT_FONT(page_title_font);
         PRINT_FONTSIZE(page_title_fontsize);
         PRINT_COLOUR(page_title_colour);
     } else {
-        printf("/show_page_title false def\n");
+        output("/show_page_title false def\n");
     }
 
     /* Map variables */
@@ -169,8 +172,8 @@ ps_map_start(void)
     PRINT_COLOUR(map_border_colour);
     PRINT_COLOUR(map_background_colour);
 
-    printf("/room_width %g def\n", room_width);
-    printf("/room_height %g def\n", room_height);
+    output("/room_width %g def\n", room_width);
+    output("/room_height %g def\n", room_height);
 
     /* Room style variables */
     ps_print_room_vars();
@@ -178,7 +181,7 @@ ps_map_start(void)
     /* Link style variables */
     ps_print_link_vars();
 
-    printf("\n%%%%EndProlog\n");
+    output("\n%%%%EndProlog\n");
 }
 
 void
@@ -197,14 +200,14 @@ ps_map_section(vhash *sect)
     /* Start a new page if required */
     if (page != ps_pagenum) {
         if (ps_pagenum > 0)
-            printf("endpage\n");
+            output("endpage\n");
 
         ps_pagenum = page;
-        printf("\n%%%%Page: %d %d\n\n", ps_pagenum, ps_pagenum);
+        output("\n%%%%Page: %d %d\n\n", ps_pagenum, ps_pagenum);
 
         rotate = (ps_rotflag ? ps_rotate : vh_iget(sect, "ROTATE"));
 
-        printf("%d %d %s beginpage\n",
+        output("%d %d %s beginpage\n",
                vh_iget(sect, "PXLEN"),
                vh_iget(sect, "PYLEN"),
                (rotate ? "true" : "false"));
@@ -212,7 +215,7 @@ ps_map_section(vhash *sect)
 
     /* Print border if required */
     if (show_map_border)
-        printf("%g %g %g %g mapborder\n",
+        output("%g %g %g %g mapborder\n",
                ps_xoff - 0.5, ps_yoff - 0.5,
                ps_xoff + xlen - 0.5, ps_yoff + ylen - 0.5);
 
@@ -220,9 +223,9 @@ ps_map_section(vhash *sect)
     if (show_map_title && vh_exists(sect, "TITLE")) {
         xpos = (double) (xlen - 1) / 2;
         ypos = (double) ylen - 1;
-        put_string("%s %g %g maptitle\n",
-                   ps_string(vh_sgetref(sect, "TITLE")),
-                   xpos + ps_xoff, ypos + ps_yoff);
+        output("%s %g %g maptitle\n",
+               ps_string(vh_sgetref(sect, "TITLE")),
+               xpos + ps_xoff, ypos + ps_yoff);
     }
 }
 
@@ -240,9 +243,9 @@ ps_map_room(vhash *room)
     /* Write coords */
     x = vh_iget(room, "X");
     y = vh_iget(room, "Y");
-    put_string("%s %g %g",
-               ps_string(vh_sgetref(room, "RDESC")),
-               x + ps_xoff, y + ps_yoff);
+    output("%s %g %g",
+           ps_string(vh_sgetref(room, "RDESC")),
+           x + ps_xoff, y + ps_yoff);
 
     /* Write item list (if any) */
     items = vh_pget(room, "ITEMS");
@@ -265,11 +268,11 @@ ps_map_room(vhash *room)
     }
 
     if (itemlist != NULL)
-        put_string(" %s true", ps_string(itemlist));
+        output(" %s true", ps_string(itemlist));
     else
-        printf(" false");
+        output(" false");
 
-    printf(" room\n");
+    output(" room\n");
 
     /* Write room exits (if any) */
     ex = vh_pget(room, "EX");
@@ -277,10 +280,8 @@ ps_map_room(vhash *room)
     if (ex != NULL) {
         double x1, y1, x2, y2;
 
-        if (px == NULL) {
-            px = vl_create();
-            py = vl_create();
-        }
+        vl_init(px);
+        vl_init(py);
 
         while (vl_length(ex) > 0) {
             vl_istore(px, 0, x);
@@ -296,7 +297,7 @@ ps_map_room(vhash *room)
             x2 = x1 + 0.35 * (x2 - x1);
             y2 = y1 + 0.35 * (y2 - y1);
 
-            printf("%g %g %g %g roomexit\n",
+            output("%g %g %g %g roomexit\n",
                    ps_xoff + x1, ps_yoff + y1,
                    ps_xoff + x2, ps_yoff + y2);
         }
@@ -357,21 +358,21 @@ ps_map_link(vhash *link)
                 angle = 90;
         }
 
-        printf("%g %g %d circle\n", xs + ps_xoff, ys + ps_yoff, angle);
+        output("%g %g %d circle\n", xs + ps_xoff, ys + ps_yoff, angle);
     } else { /* it is not a circular link */
-        printf("[");
+        output("[");
         np = vl_length(x);
         for (i = 0; i < np; i++)
-            printf(" %g %g",
+            output(" %g %g",
                    vl_dget(x, i) + ps_xoff,
                    vl_dget(y, i) + ps_yoff);
-        printf(" ]");
+        output(" ]");
 
-        printf(" %d", up);
-        printf(" %d", in);
-        printf(" %s", (oneway ? "true" : "false"));
+        output(" %d", up);
+        output(" %d", in);
+        output(" %s", (oneway ? "true" : "false"));
 
-        printf(" link\n");
+        output(" link\n");
     }
 }
 
@@ -384,7 +385,7 @@ ps_map_endsection(void)
 void
 ps_map_finish(void)
 {
-    printf("endpage\n");
+    output("endpage\n");
 }
 
 /* Print room style variables */
@@ -430,11 +431,7 @@ ps_string(char *str)
 {
     static vbuffer *b = NULL;
 
-    if (b == NULL)
-        b = vb_create();
-    else
-        vb_empty(b);
-
+    vb_init(b);
     vb_putc(b, '(');
 
     while (*str != '\0') {
