@@ -1,13 +1,13 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
  * COPYING for details.
  */
 
-/* Raw output driver */
+/* Raw output format */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -21,7 +21,8 @@
 #include <math.h>
 #include <vars.h>
 
-#include "ifm-driver.h"
+#include "ifm-format.h"
+#include "ifm-main.h"
 #include "ifm-map.h"
 #include "ifm-util.h"
 #include "ifm-raw.h"
@@ -56,31 +57,16 @@ taskfuncs raw_taskfuncs = {
 void
 raw_map_start(void)
 {
-    char *title;
-
-    if (vh_exists(map, "TITLE"))
-        title = vh_sgetref(map, "TITLE");
-    else
-        title = "Interactive Fiction map";
-
-    put_string("title: %s\n", title);
+    char *title = vh_sgetref(map, "TITLE");
+    output("title: %s\n", title);
 }
 
 void
 raw_map_section(vhash *sect)
 {
-    static int count = 0;
-    char buf[100];
-
-    count++;
-    if (!vh_exists(sect, "TITLE")) {
-        sprintf(buf, "Map section %d", count);
-        vh_sstore(sect, "TITLE", buf);
-    }
-
-    put_string("\nsection: %s\n", vh_sgetref(sect, "TITLE"));
-    printf("width: %d\n", vh_iget(sect, "XLEN"));
-    printf("height: %d\n", vh_iget(sect, "YLEN"));
+    output("\nsection: %s\n", vh_sgetref(sect, "TITLE"));
+    output("width: %d\n", vh_iget(sect, "XLEN"));
+    output("height: %d\n", vh_iget(sect, "YLEN"));
 }
 
 void
@@ -88,17 +74,16 @@ raw_map_room(vhash *room)
 {
     vlist *ex, *ey;
 
-    printf("\nroom: %d\n", vh_iget(room, "ID"));
-    put_string("name: %s\n", vh_sgetref(room, "DESC"));
-
-    printf("rpos: %d %d\n", vh_iget(room, "X"), vh_iget(room, "Y"));
+    output("\nroom: %d\n", vh_iget(room, "ID"));
+    output("name: %s\n", vh_sgetref(room, "DESC"));
+    output("rpos: %d %d\n", vh_iget(room, "X"), vh_iget(room, "Y"));
 
     ex = vh_pget(room, "EX");
     ey = vh_pget(room, "EY");
 
     if (ex != NULL && ey != NULL)
         while (vl_length(ex) > 0 && vl_length(ey) > 0)
-            printf("exit: %d %d\n", vl_ishift(ex), vl_ishift(ey));
+            output("exit: %d %d\n", vl_ishift(ex), vl_ishift(ey));
 }
 
 void
@@ -111,23 +96,23 @@ raw_map_link(vhash *link)
 
     from = vh_pget(link, "FROM");
     to = vh_pget(link, "TO");
-    printf("\nlink: %d %d\n", vh_iget(from, "ID"), vh_iget(to, "ID"));
+    output("\nlink: %d %d\n", vh_iget(from, "ID"), vh_iget(to, "ID"));
 
     x = vh_pget(link, "X");
     y = vh_pget(link, "Y");
     while (vl_length(x) > 0 && vl_length(y) > 0)
-        printf("lpos: %d %d\n", vl_ishift(x), vl_ishift(y));
+        output("lpos: %d %d\n", vl_ishift(x), vl_ishift(y));
 
     if (vh_iget(link, "ONEWAY"))
-        printf("oneway: 1\n");
+        output("oneway: 1\n");
 
     go = vh_iget(link, "GO");
     if (go != D_NONE)
-        printf("go: %s\n", dirinfo[go].sname);
+        output("go: %s\n", dirinfo[go].sname);
 
     if ((cmds = vh_pget(link, "CMD")) != NULL) {
         v_iterate(cmds, iter)
-            put_string("cmd: %s\n", vl_iter_svalref(iter));
+            output("cmd: %s\n", vl_iter_svalref(iter));
     }
 }
 
@@ -141,18 +126,18 @@ raw_map_join(vhash *join)
 
     from = vh_pget(join, "FROM");
     to = vh_pget(join, "TO");
-    printf("\njoin: %d %d\n", vh_iget(from, "ID"), vh_iget(to, "ID"));
+    output("\njoin: %d %d\n", vh_iget(from, "ID"), vh_iget(to, "ID"));
 
     if (vh_iget(join, "ONEWAY"))
-        printf("oneway: 1\n");
+        output("oneway: 1\n");
 
     go = vh_iget(join, "GO");
     if (go != D_NONE)
-        printf("go: %s\n", dirinfo[go].sname);
+        output("go: %s\n", dirinfo[go].sname);
 
     if ((cmds = vh_pget(join, "CMD")) != NULL) {
         v_iterate(cmds, iter)
-            put_string("cmd: %s\n", vl_iter_svalref(iter));
+            output("cmd: %s\n", vl_iter_svalref(iter));
     }
 }
 
@@ -167,45 +152,45 @@ raw_item_entry(vhash *item)
     vlist *list;
     viter iter;
 
-    printf("\nitem: %d\n", vh_iget(item, "ID"));
-    put_string("name: %s\n", vh_sgetref(item, "DESC"));
+    output("\nitem: %d\n", vh_iget(item, "ID"));
+    output("name: %s\n", vh_sgetref(item, "DESC"));
 
     if (vh_exists(item, "TAG"))
-        printf("tag: %s\n", vh_sgetref(item, "TAG"));
+        output("tag: %s\n", vh_sgetref(item, "TAG"));
 
     if (room != NULL)
-        printf("room: %d\n", vh_iget(room, "ID"));
+        output("room: %d\n", vh_iget(room, "ID"));
 
     if (score > 0)
-        printf("score: %d\n", score);
+        output("score: %d\n", score);
 
     if (vh_exists(item, "LEAVE"))
-        printf("leave: 1\n");
+        output("leave: 1\n");
 
     if (vh_iget(item, "HIDDEN"))
-        printf("hidden: 1\n");
+        output("hidden: 1\n");
 
     if (vh_iget(item, "FINISH"))
-        printf("finish: 1\n");
+        output("finish: 1\n");
 
     if ((list = vh_pget(item, "RTASKS")) != NULL) {
         v_iterate(list, iter) {
             task = vl_iter_pval(iter);
-            printf("after: %d\n", vh_iget(task, "ID"));
+            output("after: %d\n", vh_iget(task, "ID"));
         }
     }
 
     if ((list = vh_pget(item, "TASKS")) != NULL) {
         v_iterate(list, iter) {
             task = vl_iter_pval(iter);
-            printf("needed: %d\n", vh_iget(task, "ID"));
+            output("needed: %d\n", vh_iget(task, "ID"));
         }
     }
 
     if ((list = vh_pget(item, "NROOMS")) != NULL) {
         v_iterate(list, iter) {
             room = vl_iter_pval(iter);
-            printf("enter: %d\n", vh_iget(room, "ID"));
+            output("enter: %d\n", vh_iget(room, "ID"));
         }
     }
 
@@ -213,15 +198,15 @@ raw_item_entry(vhash *item)
         v_iterate(list, iter) {
             reach = vl_iter_pval(iter);
             room = vh_pget(reach, "FROM");
-            printf("move: %d", vh_iget(room, "ID"));
+            output("move: %d", vh_iget(room, "ID"));
             room = vh_pget(reach, "TO");
-            printf(" %d\n", vh_iget(room, "ID"));
+            output(" %d\n", vh_iget(room, "ID"));
         }
     }
 
     if (notes != NULL) {
         v_iterate(notes, iter)
-            put_string("note: %s\n", vl_iter_svalref(iter));
+            output("note: %s\n", vl_iter_svalref(iter));
     }
 }
 
@@ -235,25 +220,25 @@ raw_task_entry(vhash *task)
     int score = vh_iget(task, "SCORE");
     viter iter;
 
-    printf("\ntask: %d\n", vh_iget(task, "ID"));
-    put_string("name: %s\n", vh_sgetref(task, "DESC"));
+    output("\ntask: %d\n", vh_iget(task, "ID"));
+    output("name: %s\n", vh_sgetref(task, "DESC"));
 
     if (vh_exists(task, "TAG"))
-        printf("tag: %s\n", vh_sgetref(task, "TAG"));
+        output("tag: %s\n", vh_sgetref(task, "TAG"));
 
     if (room != NULL)
-        printf("room: %d\n", vh_iget(room, "ID"));
+        output("room: %d\n", vh_iget(room, "ID"));
 
     if (cmds != NULL) {
         v_iterate(cmds, iter)
-            put_string("cmd: %s\n", vl_iter_svalref(iter));
+            output("cmd: %s\n", vl_iter_svalref(iter));
     }
 
     if (score > 0)
-        printf("score: %d\n", score);
+        output("score: %d\n", score);
 
     if (notes != NULL) {
         v_iterate(notes, iter)
-            put_string("note: %s\n", vl_iter_svalref(iter));
+            output("note: %s\n", vl_iter_svalref(iter));
     }
 }
