@@ -1,13 +1,13 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
  * COPYING for details.
  */
 
-/* Recording output driver */
+/* Recording output format */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -19,7 +19,8 @@
 #include <string.h>
 #include <vars.h>
 
-#include "ifm-driver.h"
+#include "ifm-format.h"
+#include "ifm-main.h"
 #include "ifm-map.h"
 #include "ifm-util.h"
 #include "ifm-vars.h"
@@ -53,8 +54,13 @@ rec_task_entry(vhash *task)
 static void
 putline(char *str)
 {
-    while (*str != '\0')
-        putchar(toupper(*str++));
+    static char cmdbuf[BUFSIZ];
+    int i;
 
-    putchar('\n');
+    strcpy(cmdbuf, str);
+    
+    for (i = 0; cmdbuf[i] != '\0'; i++)
+        cmdbuf[i] = toupper(cmdbuf[i]);
+
+    output("%s\n", cmdbuf);
 }
