@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -45,9 +45,6 @@ enum {
     TS_INVALID, TS_IGNORED, TS_UNSAFE, TS_SAFE
 };
 
-/* Task step list */
-vlist *tasklist = NULL;
-
 /* Current location */
 static vhash *location = NULL;
 static char *location_desc = "nowhere";
@@ -56,6 +53,10 @@ static char *location_desc = "nowhere";
 static int all_tasks_safe = 0;
 static int keep_unused_items = 0;
 
+/* Internal counters */
+static int invertnum = 0;
+static int taskid = 0;
+
 /* Internal functions */
 static void add_task(vhash *task);
 static int do_task(vhash *task, int print, int recurse);
@@ -72,14 +73,22 @@ static int task_status(vhash *room, vhash *step);
 static int want_item(vhash *item);
 static void warn_failure(void);
 
+/* Initialize task output */
+void
+init_tasks(void)
+{
+    location = NULL;
+    location_desc = "nowhere";
+
+    taskid = 0;
+    invertnum = 0;
+}
+
 /* Add a task to the task list (if not already there) */
 static void
 add_task(vhash *task)
 {
     if (!vh_exists(task, "DEPEND")) {
-        if (tasklist == NULL)
-            tasklist = vl_create();
-
         vl_ppush(tasklist, task);
         vh_pstore(task, "DEPEND", vl_create());
     }
@@ -99,7 +108,7 @@ check_cycles(void)
     int count;
 
     /* Don't bother if no tasks */
-    if (tasklist == NULL || vl_length(tasklist) == 0)
+    if (vl_length(tasklist) == 0)
         return;
 
     /* Build task graph */
@@ -131,7 +140,7 @@ check_cycles(void)
 
         V_BUF_ADD("   cycle:\n");
         v_iterate(list, j)
-            V_BUF_ADD1("      %s\n", vl_iter_svalref(j));
+            V_BUF_ADDF("      %s\n", vl_iter_svalref(j));
     }
 
     count = vl_length(cycles);
@@ -350,7 +359,7 @@ filter_tasks(int print)
 {
     int canfilter, filter, filtered, numfiltered = 0, alldone;
     vhash *task, *otask, *item, *step;
-    char *reason;
+    char *reason = "unknown";
     vlist *list;
     viter i, j;
 
@@ -539,23 +548,22 @@ static void
 invert_items(vhash *obj, char *attr)
 {
     vlist *list, *newlist = NULL;
-    static int num = 0;
     vhash *item;
     viter iter;
 
     list = vh_pget(obj, attr);
-    num++;
+    invertnum++;
 
     if (list != NULL) {
         v_iterate(list, iter) {
             item = vl_iter_pval(iter);
-            vh_istore(item, "INVERT", num);
+            vh_istore(item, "INVERT", invertnum);
         }
     }
 
     v_iterate(items, iter) {
         item = vl_iter_pval(iter);
-        if (vh_iget(item, "INVERT") == num)
+        if (vh_iget(item, "INVERT") == invertnum)
             continue;
 
         if (newlist == NULL)
@@ -599,8 +607,7 @@ new_task(int type, vhash *data)
 {
     char *desc = vh_sgetref(data, "DESC");
     int i, score = vh_iget(data, "SCORE");
-    static int taskid = 0;
-    vhash *room, *step;
+    vhash *room = NULL, *step;
     vscalar *val;
     V_BUF_DECL;
 
@@ -610,20 +617,20 @@ new_task(int type, vhash *data)
 
     switch (type) {
     case T_MOVE:
-        V_BUF_SET1("Move to %s", desc);
+        V_BUF_SETF("Move to %s", desc);
         room = data;
         break;
     case T_GET:
-        V_BUF_SET1("Get %s", desc);
+        V_BUF_SETF("Get %s", desc);
         room = vh_pget(data, "ROOM");
         break;
     case T_DROP:
-        V_BUF_SET1("Drop %s", desc);
+        V_BUF_SETF("Drop %s", desc);
         room = NULL;
         score = 0;
         break;
     case T_GOTO:
-        V_BUF_SET1("Go to %s", desc);
+        V_BUF_SETF("Go to %s", desc);
         room = data;
         break;
     case T_USER:
@@ -635,8 +642,6 @@ new_task(int type, vhash *data)
                 vh_store(step, taskattr[i], vs_copy(val));
 
         break;
-    default:
-        fatal("internal: unknown task type");
     }
 
     if (vh_exists(data, "SAFE"))
@@ -662,7 +667,6 @@ new_task(int type, vhash *data)
 static void
 order_tasks(vhash *before, vhash *after)
 {
-    vlist *allow, *depend;
     int count = 0;
     vhash *step;
 
@@ -1068,7 +1072,7 @@ solve_game(void)
     viter iter;
 
     /* Don't bother if no tasks */
-    if (tasklist == NULL || vl_length(tasklist) == 0)
+    if (vl_length(tasklist) == 0)
         return;
 
     /* Set control variables */
@@ -1195,7 +1199,7 @@ solver_msg(int level, char *fmt, ...)
     if (TASK_VERBOSE) {
         indent(level);
         V_BUF_FMT(fmt, msg);
-        printf("%s\n", msg);
+        output("%s\n", msg);
     }
 }
 
@@ -1216,7 +1220,7 @@ task_graph(void)
     if (tasklist != NULL) {
         v_iterate(tasklist, i) {
             step = vl_iter_pval(i);
-            V_BUF_SET1("T%d", ++count);
+            V_BUF_SETF("T%d", ++count);
             vh_sstore(step, "NODE", V_BUF_VAL);
             vg_node_pstore(g, V_BUF_VAL, step);
         }
@@ -1305,15 +1309,15 @@ task_status(vhash *room, vhash *step)
 
     if (TASK_VERBOSE) {
         indent(2);
-        printf("possible: %s", vh_sgetref(step, "DESC"));
+        output("possible: %s", vh_sgetref(step, "DESC"));
 
         if (len > 0)
-            printf(" (dist %d)", len);
+            output(" (dist %d)", len);
 
         if (safemsg != NULL)
-            printf(" (unsafe: %s)", safemsg);
+            output(" (unsafe: %s)", safemsg);
 
-        printf("\n");
+        output("\n");
     }
 
     return (safemsg == NULL ? TS_SAFE : TS_UNSAFE);
@@ -1407,13 +1411,13 @@ warn_failure(void)
         if (rdesc == NULL)
             V_BUF_SET(tdesc);
         else
-            V_BUF_SET2("%s (%s)", tdesc, rdesc);
+            V_BUF_SETF("%s (%s)", tdesc, rdesc);
 
         vl_spush(list, V_BUF_VAL);
     }
 
     /* Build warning message */
-    V_BUF_SET1("   final location:\n      %s\n",
+    V_BUF_SETF("   final location:\n      %s\n",
                vh_sgetref(location, "DESC"));
 
     keys = vh_keys(reasons);
@@ -1425,10 +1429,10 @@ warn_failure(void)
         entries = vl_join(list, ", ");
 
         list = vl_filltext(entries, 65);
-        V_BUF_ADD1("   %s:\n", reason);
+        V_BUF_ADDF("   %s:\n", reason);
 
         v_iterate(list, iter)
-            V_BUF_ADD1("      %s\n", vl_iter_svalref(iter));
+            V_BUF_ADDF("      %s\n", vl_iter_svalref(iter));
     }
 
     vl_destroy(keys);
