@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -19,10 +19,8 @@ enum {
     T_MOVE, T_GET, T_DROP, T_GOTO, T_USER
 };
 
-/* Task list */
-extern vlist *tasklist;
-
 /* Advertised functions */
+extern void init_tasks(void);
 extern void check_cycles(void);
 extern vhash *require_task(vhash *step);
 extern void setup_tasks(void);
