@@ -1,13 +1,13 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
  * COPYING for details.
  */
 
-/* Text output driver */
+/* Text output format */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -21,13 +21,34 @@
 #include <math.h>
 #include <vars.h>
 
-#include "ifm-driver.h"
+#include "ifm-format.h"
+#include "ifm-main.h"
 #include "ifm-map.h"
 #include "ifm-task.h"
 #include "ifm-util.h"
 #include "ifm-vars.h"
 #include "ifm-text.h"
 
+/* Internal functions */
+static void text_init(void);
+
+/* Output function list */
+outputfuncs text_outputfuncs = {
+    text_init,
+    NULL
+};
+
+/* Map function list */
+mapfuncs text_mapfuncs = {
+    text_map_start,
+    text_map_section,
+    text_map_room,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+};
+
 /* Item function list */
 itemfuncs text_itemfuncs = {
     NULL,
@@ -48,95 +69,129 @@ static int total = 0;
 /* Total distance travelled */
 static int travel = 0;
 
+/* Counts of things */
+static int taskcount = 0;
+static int itemcount = 0;
+static int sectcount = 0;
+
+/* Task settings */
+static vhash *lastroom = NULL;
+static int moved = 0;
+
+/* Map functions */
+void
+text_map_start(void)
+{
+    char *title = vh_sgetref(map, "TITLE");
+    output("Map sections for %s\n", title);
+}
+
+void
+text_map_section(vhash *sect)
+{
+    char *title = vh_sgetref(sect, "TITLE");
+    vlist *rooms = vh_pget(sect, "ROOMS");
+    int width = vh_iget(sect, "XLEN");
+    int height = vh_iget(sect, "YLEN");
+
+    output("\n%d (%s) (size: %dx%d, rooms: %d):\n",
+           ++sectcount, title, width, height, vl_length(rooms));
+}
+
+void
+text_map_room(vhash *room)
+{
+    output("   %s (%d, %d)\n",
+           vh_sgetref(room, "DESC"),
+           vh_iget(room, "X"),
+           vh_iget(room, "Y"));
+}
+
 /* Item functions */
 void
 text_item_entry(vhash *item)
 {
     vlist *notes = vh_pget(item, "NOTE");
     vhash *room, *task, *reach;
-    static int count = 0;
     vlist *list;
     char *title;
     V_BUF_DECL;
     viter iter;
 
-    if (count++ == 0) {
-        if (vh_exists(map, "TITLE"))
-            title = vh_sgetref(map, "TITLE");
-        else
-            title = "Interactive Fiction game";
-        put_string("Item list for %s\n", title);
+    if (itemcount++ == 0) {
+        title = vh_sgetref(map, "TITLE");
+        output("Item list for %s\n", title);
     }
 
-    put_string("\n%s:\n", vh_sgetref(item, "DESC"));
+    output("\n%s:\n", vh_sgetref(item, "DESC"));
 
     if ((room = vh_pget(item, "ROOM")) == NULL)
-        printf("   carried at the start of the game\n");
+        output("   carried at the start of the game\n");
     else
-        printf("   %s in %s\n",
+        output("   %s in %s\n",
                (vh_iget(item, "HIDDEN") ? "hidden" : "seen"),
                vh_sgetref(room, "DESC"));
 
     if (vh_exists(item, "SCORE"))
-        printf("   scores %d points when picked up\n",
+        output("   scores %d points when picked up\n",
                vh_iget(item, "SCORE"));
 
     if (vh_exists(item, "LEAVE"))
-        printf("   may have to be dropped when moving\n");
+        output("   may have to be dropped when moving\n");
 
     if (vh_exists(item, "FINISH"))
-        printf("   finishes the game when picked up\n");
+        output("   finishes the game when picked up\n");
 
     if ((list = vh_pget(item, "RTASKS")) != NULL) {
-        printf("   obtained after:\n");
+        output("   obtained after:\n");
         v_iterate(list, iter) {
             task = vl_iter_pval(iter);
             if ((room = vh_pget(task, "ROOM")) == NULL)
                 V_BUF_SET(vh_sgetref(task, "DESC"));
             else
-                V_BUF_SET2("%s (%s)",
+                V_BUF_SETF("%s (%s)",
                            vh_sgetref(task, "DESC"),
                            vh_sgetref(room, "DESC"));
-            put_string("      %s\n", V_BUF_VAL);
+            output("      %s\n", V_BUF_VAL);
         }
     }
 
     if ((list = vh_pget(item, "TASKS")) != NULL) {
-        printf("   needed for:\n");
+        output("   needed for:\n");
         v_iterate(list, iter) {
             task = vl_iter_pval(iter);
             if ((room = vh_pget(task, "ROOM")) == NULL)
                 V_BUF_SET(vh_sgetref(task, "DESC"));
             else
-                V_BUF_SET2("%s (%s)",
+                V_BUF_SETF("%s (%s)",
                            vh_sgetref(task, "DESC"),
                            vh_sgetref(room, "DESC"));
-            put_string("      %s\n", V_BUF_VAL);
+            output("      %s\n", V_BUF_VAL);
         }
     }
 
     if ((list = vh_pget(item, "NROOMS")) != NULL) {
-        printf("   needed to enter:\n");
+        output("   needed to enter:\n");
         v_iterate(list, iter) {
             room = vl_iter_pval(iter);
-            put_string("      %s\n", vh_sgetref(room, "DESC"));
+            output("      %s\n", vh_sgetref(room, "DESC"));
         }
     }
 
     if ((list = vh_pget(item, "NLINKS")) != NULL) {
-        printf("   needed to move:\n");
+        output("   needed to move:\n");
         v_iterate(list, iter) {
             reach = vl_iter_pval(iter);
             room = vh_pget(reach, "FROM");
-            put_string("      %s to", vh_sgetref(room, "DESC"));
+            output("      %s to", vh_sgetref(room, "DESC"));
             room = vh_pget(reach, "TO");
-            put_string(" %s\n", vh_sgetref(room, "DESC"));
+            output(" %s\n", vh_sgetref(room, "DESC"));
         }
     }
 
     if (notes != NULL) {
         v_iterate(notes, iter)
-            put_string("   note: %s\n", vl_iter_svalref(iter));
+            output("   note: %s\n", vl_iter_svalref(iter));
     }
 }
 
@@ -148,52 +203,46 @@ text_task_entry(vhash *task)
     vlist *notes = vh_pget(task, "NOTE");
     vhash *room = vh_pget(task, "ROOM");
     vlist *cmds = vh_pget(task, "CMD");
-    static vhash *lastroom = NULL;
-    static int moved = 0;
-    static int count = 0;
     int type, score;
     vhash *otask;
     char *title;
     viter iter;
 
-    if (count == 0) {
-        if (vh_exists(map, "TITLE"))
-            title = vh_sgetref(map, "TITLE");
-        else
-            title = "Interactive Fiction game";
-        put_string("Task list for %s\n", title);
+    if (taskcount == 0) {
+        title = vh_sgetref(map, "TITLE");
+        output("Task list for %s\n", title);
     }
 
     type = vh_iget(task, "TYPE");
 
     if (type == T_MOVE) {
-        if (count == 0)
-            put_string("\nStart: %s\n", vh_sgetref(startroom, "DESC"));
+        if (taskcount == 0)
+            output("\nStart: %s\n", vh_sgetref(startroom, "DESC"));
 
         if (!moved)
-            printf("\n");
+            output("\n");
 
-        put_string("%s", vh_sgetref(task, "DESC"));
+        output("%s", vh_sgetref(task, "DESC"));
         if (cmds != NULL)
-            put_string(" (%s)", vl_join(cmds, ". "));
+            output(" (%s)", vl_join(cmds, ". "));
 
-        printf("\n");
+        output("\n");
         travel++;
         moved++;
     } else {
         if (room != NULL && (moved || room != lastroom))
-            put_string("\n%s:\n", vh_sgetref(room, "DESC"));
-        else if (!count)
-            printf("\nFirstly:\n");
+            output("\n%s:\n", vh_sgetref(room, "DESC"));
+        else if (!taskcount)
+            output("\nFirstly:\n");
 
-        put_string("   %s\n", vh_sgetref(task, "DESC"));
+        output("   %s\n", vh_sgetref(task, "DESC"));
 
         if (cmds != NULL) {
             if (vl_length(cmds) > 0) {
                 v_iterate(cmds, iter)
-                    put_string("      cmd: %s\n", vl_iter_svalref(iter));
+                    output("      cmd: %s\n", vl_iter_svalref(iter));
             } else {
-                printf("      no action required\n");
+                output("      no action required\n");
             }
         }
 
@@ -205,25 +254,25 @@ text_task_entry(vhash *task)
             otask = vl_iter_pval(iter);
 
             if (type != T_MOVE)
-                printf("   ");
+                output("   ");
 
-            put_string("   also does: %s\n", vh_sgetref(otask, "DESC"));
+            output("   also does: %s\n", vh_sgetref(otask, "DESC"));
         }
     }
 
     if ((score = vh_iget(task, "SCORE")) > 0) {
         if (type != T_MOVE)
-            printf("   ");
+            output("   ");
 
-        printf("   score: %d\n", score);
+        output("   score: %d\n", score);
     }
 
     if (notes != NULL) {
         v_iterate(notes, iter) {
             if (type != T_MOVE)
-                printf("   ");
+                output("   ");
 
-            put_string("   note: %s\n", vl_iter_svalref(iter));
+            output("   note: %s\n", vl_iter_svalref(iter));
         }
     }
 
@@ -231,14 +280,29 @@ text_task_entry(vhash *task)
         lastroom = room;
 
     total += score;
-    count++;
+    taskcount++;
 }
 
 void
 text_task_finish(void)
 {
     if (travel > 0)
-        printf("\nTotal distance travelled: %d\n", travel);
+        output("\nTotal distance travelled: %d\n", travel);
     if (total > 0)
-        printf("\nTotal score: %d\n", total);
+        output("\nTotal score: %d\n", total);
+}
+
+/* Initialise text output */
+static void
+text_init(void)
+{
+    total = 0;
+    travel = 0;
+    moved = 0;
+
+    taskcount = 0;
+    itemcount = 0;
+    sectcount = 0;
+
+    lastroom = NULL;
 }
