@@ -1,17 +1,19 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
  * COPYING for details.
  */
 
-/* Tcl/Tk output driver header */
+/* Tcl/Tk output format header */
 
 #ifndef IFM_TK_H
 #define IFM_TK_H
 
+#include "ifm-format.h"
+
 extern mapfuncs tk_mapfuncs;
 extern itemfuncs tk_itemfuncs;
 extern taskfuncs tk_taskfuncs;
