@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -20,7 +20,7 @@
 #include <math.h>
 #include <vars.h>
 
-#include "ifm-driver.h"
+#include "ifm-format.h"
 #include "ifm-main.h"
 #include "ifm-map.h"
 #include "ifm-util.h"
@@ -96,7 +96,7 @@ find_file(char *name)
     viter iter;
 
     v_iterate(ifm_search, iter) {
-        path = V_BUF_SET2("%s/%s", vl_iter_svalref(iter), name);
+        path = V_BUF_SETF("%s/%s", vl_iter_svalref(iter), name);
         if (v_exists(path))
             return path;
     }
@@ -140,8 +140,8 @@ indent(int num)
 {
     int i;
 
-    for (i = 0; i < 4 * num; i++)
-        putchar(' ');
+    for (i = 0; i < num; i++)
+        output("    ");
 }
 
 /* Warn about obsolete syntax */
@@ -345,22 +345,11 @@ pack_sections(int xmax, int ymax)
     return num;
 }
 
-/* Send a string to stdout */
-void
-put_string(char *fmt, ...)
-{
-    V_BUF_DECL;
-    char *str;
-
-    V_BUF_FMT(fmt, str);
-    fputs(str, stdout);
-}
-
 /* Set up room names */
 void
 setup_room_names(void)
 {
-    char tag[10], *name, *jstyle = var_string("join_format");
+    char tag[20], *name, *jstyle = var_string("join_format");
     vhash *room, *join, *from, *to;
     int jnum = 0;
     V_BUF_DECL;
@@ -414,7 +403,7 @@ setup_room_names(void)
         v_iterate(rooms, iter) {
             room = vl_iter_pval(iter);
             if (vh_exists(room, "TAG")) {
-                V_BUF_SET2("%s [%s]",
+                V_BUF_SETF("%s [%s]",
                            vh_sgetref(room, "RDESC"),
                            vh_sgetref(room, "TAG"));
                 vh_sstore(room, "RDESC", V_BUF_VAL);
@@ -427,7 +416,7 @@ setup_room_names(void)
 vlist *
 split_line(char *string, double ratio)
 {
-    int llen = (int) (sqrt((double) strlen(string)) * ratio);
+    unsigned int llen = (int) (sqrt((double) strlen(string)) * ratio);
     vlist *list = vl_create();
     char *tok, *val;
     V_BUF_DECL;
@@ -440,7 +429,7 @@ split_line(char *string, double ratio)
             vl_spush(list, val);
             val = V_BUF_SET(tok);
         } else {
-            val = V_BUF_ADD(" ");
+            V_BUF_ADD(" ");
             val = V_BUF_ADD(tok);
         }
     }
