@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -24,14 +24,9 @@
 #include "ifm-util.h"
 #include "ifm-vars.h"
 
-#define INIT_VARS                                                       \
-        if (nvars == NULL) cvars = nvars = vh_create();                 \
-        if (styles == NULL) styles = vh_create();                       \
-        if (rstyles == NULL) rstyles = vh_create()
-
 #define VAR_CHECK(id, var)                                              \
         if ((var = var_get(id)) == NULL)                                \
-            fatal("variable '%s' is not defined", id)
+            err("variable '%s' is not defined", id)
 
 #define ALPHA(c) (c == '_' || isalpha(c))
 #define ALNUM(c) (c == '_' || isalnum(c))
@@ -51,11 +46,39 @@ static vhash *rstyles = NULL;
 /* Style list */
 static vlist *style_list = NULL;
 
+/* Current variable values */
+static vhash *values = NULL;
+
 /* Internal functions */
 static vhash *read_colour_defs(FILE *fp);
-static char *var_encode(char *driver, char *var);
+static char *var_encode(char *format, char *var);
 static void var_print(vhash *vars, char *style);
 
+/* Initialise variables */
+void
+init_vars(void)
+{
+    if (nvars != NULL)
+        v_destroy(nvars);
+
+    if (styles != NULL)
+        v_destroy(styles);
+
+    if (rstyles != NULL)
+        v_destroy(rstyles);
+
+    cvars = nvars = vh_create();
+    styles = vh_create();
+    rstyles = vh_create();
+
+    if (style_list != NULL)
+        vl_destroy(style_list);
+
+    style_list = NULL;
+
+    vh_init(values);
+}
+
 /* Add a style to the style list */
 void
 add_style(char *name)
@@ -104,8 +127,8 @@ load_styles(void)
         if (vh_exists(styles, name))
             continue;
 
-        V_BUF_SET1("%s.ifm", name);
-        parse_input(V_BUF_VAL, 1, 0);
+        V_BUF_SETF("%s.ifm", name);
+        read_input(V_BUF_VAL, 1, 0);
 
         if (!vh_exists(styles, name))
             warn("style '%s' referenced but not defined", name);
@@ -168,7 +191,7 @@ read_colour_defs(FILE *fp)
             continue;
 
         /* Scale RGB values */
-        V_BUF_SET3("%.3g %.3g %.3g",
+        V_BUF_SETF("%.3g %.3g %.3g",
                    red / 255.0, green / 255.0, blue / 255.0);
 
         /* Add colour */
@@ -182,7 +205,6 @@ read_colour_defs(FILE *fp)
 void
 ref_style(char *name)
 {
-    INIT_VARS;
     vh_istore(rstyles, name, 1);
 }
 
@@ -190,8 +212,6 @@ ref_style(char *name)
 void
 set_style(char *name)
 {
-    INIT_VARS;
-
     if (name != NULL && strlen(name) > 0)
         cvars = vh_add_hash(styles, name);
     else
@@ -202,20 +222,16 @@ set_style(char *name)
 void
 set_style_list(vlist *list)
 {
-    style_list = list;
+    style_list = list ? vl_copy(list) : NULL;
 }
 
 /* Return whether a variable has changed since last accessed */
 int
 var_changed(char *id)
 {
-    static vhash *values = NULL;
     vscalar *sval, *cval;
     int changed = 1;
 
-    if (values == NULL)
-        values = vh_create();
-
     sval = var_get(id);
 
     if ((cval = vh_get(values, id)) == NULL && sval == NULL)
@@ -236,7 +252,7 @@ var_colour(char *id)
     static vhash *defs = NULL;
     char *name, *file, *path;
     double red, green, blue;
-    FILE *fp;
+    FILE *fp = NULL;
 
     /* Get colour name */
     name = var_string(id);
@@ -250,35 +266,42 @@ var_colour(char *id)
         file = var_string("colour_file");
 
         if ((path = find_file(file)) == NULL)
-            fatal("can't locate colour definitions file '%s'", file);
+            err("can't locate colour definitions file '%s'", file);
 
-        if ((fp = fopen(path, "r")) == NULL)
-            fatal("can't open '%s'", path);
+        if (path != NULL && (fp = fopen(path, "r")) == NULL)
+            err("can't open '%s'", path);
 
-        defs = read_colour_defs(fp);
-        fclose(fp);
+        if (fp != NULL) {
+            defs = read_colour_defs(fp);
+            fclose(fp);
+        }
     }
 
     /* Look up colour name */
-    if (!vh_exists(defs, name))
-        fatal("colour '%s' is not defined", name);
+    if (defs == NULL)
+        return id;
+
+    if (defs != NULL && !vh_exists(defs, name)) {
+        err("colour '%s' is not defined", name);
+        return id;
+    }
 
     return vh_sgetref(defs, name);
 }
 
 /* Encode a variable */
 static char *
-var_encode(char *driver, char *var)
+var_encode(char *format, char *var)
 {
     V_BUF_DECL;
 
-    if (driver == NULL)
-        driver = "global";
+    if (format == NULL)
+        format = "global";
 
     if (strchr(var, '.') != NULL)
         V_BUF_SET(var);
     else
-        V_BUF_SET2("%s.%s", driver, var);
+        V_BUF_SETF("%s.%s", format, var);
 
     return V_BUF_VAL;
 }
@@ -292,8 +315,6 @@ var_get(char *id)
     vhash *svars;
     int i, len;
 
-    INIT_VARS;
-
     /* Check style list if required */
     if (style_list != NULL) {
         len = vl_length(style_list);
@@ -358,9 +379,7 @@ var_list(void)
     char *style;
     viter iter;
 
-    INIT_VARS;
-
-    printf("# IFM defined variables.\n");
+    output("# IFM defined variables.\n");
 
     /* Non-style variables */
     var_print(nvars, NULL);
@@ -390,16 +409,16 @@ var_print(vhash *vars, char *style)
     names = vh_keys(vars);
     vl_sort(names, NULL);
 
-    printf("\n");
+    output("\n");
 
     if (style != NULL)
-        printf("# Style '%s' variables.\n", style);
+        output("# Style '%s' variables.\n", style);
     else
-        printf("# General variables.\n");
+        output("# General variables.\n");
 
     v_iterate(names, iter) {
         name = vl_iter_svalref(iter);
-        printf("%s = ", name);
+        output("%s = ", name);
         val = vh_get(vars, name);
 
         switch (vs_type(val)) {
@@ -418,14 +437,14 @@ var_print(vhash *vars, char *style)
             break;
 
         default:
-            printf("%s", vs_sget(val));
+            output("%s", vs_sget(val));
             break;
         }
 
         if (style != NULL)
-            printf(" in style %s", style);
+            output(" in style %s", style);
 
-        printf(";\n");
+        output(";\n");
     }
 
     vl_destroy(names);
@@ -442,13 +461,11 @@ var_real(char *id)
 
 /* Set a scalar variable */
 void
-var_set(char *driver, char *id, vscalar *val)
+var_set(char *format, char *id, vscalar *val)
 {
     char *key;
 
-    INIT_VARS;
-
-    key = var_encode(driver, id);
+    key = var_encode(format, id);
 
     if (val != NULL)
         vh_store(cvars, key, val);
