@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -15,6 +15,7 @@
 #define VAR_DEF(var)  (var_get(var) != NULL)
 
 /* Advertised functions */
+extern void init_vars(void);
 extern void add_style(char *name);
 extern char *current_style(void);
 extern vlist *current_styles(void);
@@ -30,7 +31,7 @@ extern vscalar *var_get(char *id);
 extern int var_int(char *id);
 extern void var_list(void);
 extern double var_real(char *id);
-extern void var_set(char *driver, char *id, vscalar *val);
+extern void var_set(char *format, char *id, vscalar *val);
 extern char *var_string(char *id);
 
 #endif
