@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -14,6 +14,8 @@
 
 #include <vars.h>
 
-extern void fig_write_figure(vhash *figure, FILE *fp);
+extern void fig_set_stream(FILE *fp);
+extern void fig_set_handler(void (*func)(char *line));
+extern void fig_write_figure(vhash *figure);
 
 #endif
