@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -65,9 +65,7 @@
 #include "vars-macros.h"
 #include "vars-memory.h"
 
-/*@-noparams@*/
 #include "getopt.h"
-/*@=noparams@*/
 
 #define SET_VAR(type, var, val)                                         \
         if (var != NULL) *((type *) var) = val
@@ -79,6 +77,9 @@
         option(letter, name, otype, argname, vtype, var, desc);         \
 }
 
+/* All option data */
+static vlist *option_data = NULL;
+
 /* Option lists */
 static vlist *option_list = NULL;
 
@@ -97,9 +98,9 @@ static void option(char optletter, char *optname, enum v_oflag type,
                    char *argname, int vtype, void *var, char *fmt, ...);
 static char *optstring(void);
 static int set_long_option(vhash *opts, char *opt, char *arg);
+static int set_short_option(vhash *opts, char opt, char *arg);
 static int set_option(vhash *opts, char *name, enum v_oflag type, void *var,
                       enum v_stype vtype, char *arg);
-static int set_short_option(vhash *opts, char opt, char *arg);
 
 /*!
   @brief   Parse command-line options.
@@ -161,7 +162,7 @@ v_getopts(int argc, char *argv[])
     vh_pstore(opts, "ARGS", args);
 
     /* Parse arguments */
-    if (long_optlist == NULL) {
+    if (long_optlist == NULL || vl_length(long_optlist) == 0) {
         /* Single-letter options only */
         while ((c = getopt(argc, argv, optstring())) != EOF) {
             if (c == '?')
@@ -489,14 +490,14 @@ v_usage(char *fmt, ...)
                 continue;
 
             if (strlen(sname) == 0) {
-                if (short_optlist == NULL)
-                    V_BUF_SET1("-%s", lname);    
+                if (vl_length(short_optlist) == 0)
+                    V_BUF_SETF("--%s", lname);    
                 else
-                    V_BUF_SET1("    -%s", lname);
+                    V_BUF_SETF("    --%s", lname);
             } else if (strlen(lname) == 0) {
-                V_BUF_SET1("-%s", sname);
+                V_BUF_SETF("-%s", sname);
             } else {
-                V_BUF_SET2("-%s, -%s", sname, lname);
+                V_BUF_SETF("-%s, --%s", sname, lname);
             }
 
             arg = 1;
@@ -523,7 +524,7 @@ v_usage(char *fmt, ...)
             vh_sstore(entry, "OPTS", V_BUF_VAL);
 
             if (arg) {
-                V_BUF_SET3("%s%s%s",
+                V_BUF_SETF("%s%s%s",
                            b1, (argname != NULL ? argname : "arg"), b2);
                 vh_sstore(entry, "ARG", V_BUF_VAL);
             }
@@ -540,7 +541,7 @@ v_usage(char *fmt, ...)
         str = vh_sgetref(entry, "OPTS");
 
         if (vh_exists(entry, "ARG"))
-            str = V_BUF_SET2("%s%s", str, vh_sgetref(entry, "ARG"));
+            str = V_BUF_SETF("%s%s", str, vh_sgetref(entry, "ARG"));
 
         vh_sstore(entry, "NAME", str);
         if (strlen(str) > (size_t) len)
@@ -552,7 +553,7 @@ v_usage(char *fmt, ...)
     puts(str);
 
     /* Build format string */
-    V_BUF_SET1("   %%-%ds %%s\n", len + 1);
+    V_BUF_SETF("   %%-%ds %%s\n", len + 1);
 
     /* Print options */
     v_iterate(entries, iter) {
@@ -565,7 +566,42 @@ v_usage(char *fmt, ...)
         for (i = 0; i < vl_length(list); i++)
             printf(V_BUF_VAL, (i > 0 ? "" : vh_sgetref(entry, "NAME")),
                    vl_sgetref(list, i));
+
+        vl_destroy(list);
     }
+
+    /* Clean up */
+    v_destroy(entries);
+}
+
+/*!
+  @brief   Initialise options.
+  @ingroup options_parse
+
+  Reinitialize the option declarations.  This is only needed if you intend
+  to set up option parsing more than once.
+*/
+void
+v_initopts(void)
+{
+    if (option_data != NULL)
+        v_destroy(option_data);
+
+    option_list = vl_create();
+    short_optlist = vl_create();
+    long_optlist = vl_create();
+    short_opthash = vh_create();
+    long_opthash = vh_create();
+
+    option_data = vl_list(V_TYPE_POINTER, option_list,
+                          V_TYPE_POINTER, short_optlist,
+                          V_TYPE_POINTER, long_optlist,
+                          V_TYPE_POINTER, short_opthash,
+                          V_TYPE_POINTER, long_opthash,
+                          V_TYPE_NULL);
+
+    optgroup = NULL;
+    optind = 1;
 }
 
 /*!
@@ -617,9 +653,8 @@ option(char optletter, char *optname, enum v_oflag type, char *argname,
     char *desc = NULL;
     V_BUF_DECL;
 
-    /* Initialise */
-    if (option_list == NULL)
-        option_list = vl_create();
+    if (option_data == NULL)
+        v_initopts();
 
     if (optletter == '\0' && optname == NULL)
         v_fatal("v_option(): no option letter or long name specified");
@@ -653,12 +688,7 @@ option(char optletter, char *optname, enum v_oflag type, char *argname,
 
     /* Add single-letter option if required */
     if (optletter != '\0') {
-        if (short_optlist == NULL) {
-            short_optlist = vl_create();
-            short_opthash = vh_create();
-        }
-
-        V_BUF_SET1("%c", optletter);
+        V_BUF_SETF("%c", optletter);
         vh_sstore(opt, "SHORT", V_BUF_VAL);
 
         if (vh_exists(short_opthash, V_BUF_VAL))
@@ -673,14 +703,9 @@ option(char optletter, char *optname, enum v_oflag type, char *argname,
         if (strlen(optname) < (size_t) 2)
             v_fatal("v_option(): long name must be at least 2 characters");
 
-        if (long_optlist == NULL) {
-            long_optlist = vl_create();
-            long_opthash = vh_create();
-        }
-
         vh_sstore(opt, "LONG", optname);
 
-        if (vh_exists(long_opthash, V_BUF_VAL))
+        if (vh_exists(long_opthash, optname))
             v_fatal("v_option(): option '%s' declared twice", optname);
 
         vl_ppush(long_optlist, opt);
@@ -698,7 +723,7 @@ optstring(void)
     V_BUF_DECL;
     int type;
 
-    if (short_optlist == NULL)
+    if (vl_length(short_optlist) == 0)
         return "";
 
     V_BUF_INIT;
@@ -757,6 +782,35 @@ set_long_option(vhash *opts, char *opt, char *arg)
     return 1;
 }
 
+/* Set a short option */
+static int
+set_short_option(vhash *opts, char opt, char *arg)
+{
+    int type, vtype;
+    vhash *option;
+    V_BUF_DECL;
+    char *name;
+    void *var;
+
+    /* Set the option */
+    V_BUF_SETF("%c", opt);
+    option = vh_pget(short_opthash, V_BUF_VAL);
+    name = vh_sgetref(option, "SHORT");
+    type = vh_iget(option, "TYPE");
+    var = vh_pget(option, "VAR");
+    vtype = vh_iget(option, "VTYPE");
+
+    if (!set_option(opts, name, type, var, vtype, arg))
+        return 0;
+
+    /* Set equivalent long option as well */
+    name = vh_sgetref(option, "LONG");
+    if (strlen(name) > 0 && !set_option(opts, name, type, NULL, vtype, arg))
+        return 0;
+
+    return 1;
+}
+
 /* Set an option */
 static int
 set_option(vhash *opts, char *name, enum v_oflag type,
@@ -851,32 +905,3 @@ set_option(vhash *opts, char *name, enum v_oflag type,
 
     return 1;
 }
-
-/* Set a short option */
-static int
-set_short_option(vhash *opts, char opt, char *arg)
-{
-    int type, vtype;
-    vhash *option;
-    V_BUF_DECL;
-    char *name;
-    void *var;
-
-    /* Set the option */
-    V_BUF_SET1("%c", opt);
-    option = vh_pget(short_opthash, V_BUF_VAL);
-    name = vh_sgetref(option, "SHORT");
-    type = vh_iget(option, "TYPE");
-    var = vh_pget(option, "VAR");
-    vtype = vh_iget(option, "VTYPE");
-
-    if (!set_option(opts, name, type, var, vtype, arg))
-        return 0;
-
-    /* Set equivalent long option as well */
-    name = vh_sgetref(option, "LONG");
-    if (strlen(name) > 0 && !set_option(opts, name, type, NULL, vtype, arg))
-        return 0;
-
-    return 1;
-}
