@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -81,7 +81,6 @@
 #include "vars-hash.h"
 #include "vars-macros.h"
 #include "vars-memory.h"
-#include "vars-yaml.h"
 
 /* Calculate a hash value */
 #define VH_HASHVAL(size, key, value) do {       \
@@ -149,7 +148,7 @@ typedef struct v_table {
 } vtable;
 
 /* Type variable */
-vtype *vhash_type = NULL;
+static vtype *vhash_type = NULL;
 
 /* List of primes closest to powers of 2 */
 static int primes[] = {
@@ -172,8 +171,6 @@ char vh_keybuf[V_HEXSTRING_SIZE];
 /* Internal functions */
 static vtable **vh_entries(vhash *h);
 static int vh_entries_cmp(vtable **t1, vtable **t2);
-static int vh_xmldump(vhash *h, FILE *fp);
-static int vh_yamldump(vhash *h, FILE *fp);
 
 /*!
   @brief   Create abbreviation table from word list.
@@ -389,8 +386,6 @@ vh_declare(void)
         v_freeze_func(vhash_type, vh_freeze);
         v_thaw_func(vhash_type, (void *(*)()) vh_thaw);
         v_print_func(vhash_type, vh_print);
-        v_xmldump_func(vhash_type, vh_xmldump);
-        v_yamldump_func(vhash_type, vh_yamldump);
         v_destroy_func(vhash_type, vh_destroy);
         v_traverse_func(vhash_type, vh_traverse);
     }
@@ -1231,59 +1226,3 @@ vh_write(vhash *h, FILE *fp)
 
     return 1;
 }
-
-/* Dump XML contents of a hash */
-static int
-vh_xmldump(vhash *h, FILE *fp)
-{
-    vtable **entries;
-    vscalar *val;
-    char *key;
-    int i;
-
-    VH_CHECK(h);
-
-    v_xmldump_start(fp);
-    entries = vh_entries(h);
-
-    for (i = 0; entries[i] != NULL; i++) {
-        key = entries[i]->key;
-        val = entries[i]->val;
-
-        if (vs_defined(val)) {
-            v_xmldump_tag_start(fp, "entry", "key", key, NULL);
-
-            if (vs_xmldump(val, fp))
-                v_xmldump_tag_finish(fp, "entry");
-            else
-                return 0;
-        } else {
-            v_xmldump_tag(fp, "entry", "key", key, NULL);
-        }
-    }
-
-    V_DEALLOC(entries);
-    v_xmldump_finish(fp);
-
-    return 1;
-}
-
-/* Dump contents of a hash in YAML format */
-static int
-vh_yamldump(vhash *h, FILE *fp)
-{
-    int i;
-
-    VH_CHECK(h);
-
-    if (!v_yaml_start(fp))
-        return 0;
-
-    if (!v_yaml_write_hash(h, fp))
-        return 0;
-
-    if (!v_yaml_finish(fp))
-        return 0;
-
-    return 1;
-}
