@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -54,10 +54,9 @@
 #include "vars-list.h"
 #include "vars-macros.h"
 #include "vars-memory.h"
-#include "vars-yaml.h"
 
 #ifndef LIST_DEFAULT_SIZE
-#define LIST_DEFAULT_SIZE 16
+#define LIST_DEFAULT_SIZE 4
 #endif
 
 #define SENTENCE_END ".!?"
@@ -102,13 +101,11 @@ struct v_list {
 };
 
 /* Type variable */
-vtype *vlist_type = NULL;
+static vtype *vlist_type = NULL;
 
 /* Internal functions */
 static void vl_need_push(vlist *l, int num);
 static void vl_need_unshift(vlist *l, int num);
-static int vl_xmldump(vlist *l, FILE *fp);
-static int vl_yamldump(vlist *l, FILE *fp);
 
 /*!
   @brief   Append a list to another list.
@@ -261,8 +258,6 @@ vl_declare(void)
         v_print_func(vlist_type, vl_print);
         v_destroy_func(vlist_type, vl_destroy);
         v_traverse_func(vlist_type, vl_traverse);
-        v_xmldump_func(vlist_type, vl_xmldump);
-        v_yamldump_func(vlist_type, vl_yamldump);
     }
 
     return vlist_type;
@@ -473,12 +468,12 @@ vl_insert(vlist *l, int num, vscalar *s)
     }
 
     vl_need_push(l, 1);
-    l->end++;
 
     for (i = LLEN(l) - 1; i >= num; i--)
         LVAL(l, i + 1) = LVAL(l, i);
 
     LVAL(l, num) = s;
+    l->end++;
 
     return num;
 }
@@ -904,10 +899,10 @@ vl_remove(vlist *l, int num)
     }
 
     s = LVAL(l, num);
+    l->end--;
 
     for (i = num; i < LLEN(l); i++)
         LVAL(l, i) = LVAL(l, i + 1);
-    l->end--;
 
     return s;
 }
@@ -1020,7 +1015,7 @@ vl_sort(vlist *l, int (*compare)(vscalar **s1, vscalar **s2))
 	compare = vs_cmp;
 
     if (LLEN(l) > 1)
-        qsort(l->list, (size_t) LLEN(l), sizeof(vscalar *),
+        qsort(l->list + l->beg, (size_t) LLEN(l), sizeof(vscalar *),
               (int (*)(const void *, const void *)) compare);
 }
 
@@ -1113,7 +1108,7 @@ vl_splice(vlist *l, int offset, int length, vlist *r)
 vlist *
 vl_split_string(char *string, char *sep, char *quotes, int nullfields)
 {
-    char openquote, closequote, *str = string, *cp;
+    char openquote = '\0', closequote = '\0', *str = string, *cp;
     int addword = 0;
     vscalar *val;
     vlist *l;
@@ -1353,46 +1348,3 @@ vl_write(vlist *l, FILE *fp)
 
     return 1;
 }
-
-/* Dump contents of a list in XML format */
-static int
-vl_xmldump(vlist *l, FILE *fp)
-{
-    int i;
-
-    VL_CHECK(l);
-
-    v_xmldump_start(fp);
-
-    for (i = l->beg; i <= l->end; i++) {
-        v_xmldump_tag_start(fp, "entry", NULL);
-
-        if (!vs_xmldump(l->list[i], fp))
-            return 0;
-
-        v_xmldump_tag_finish(fp, "entry");
-    }
-
-    v_xmldump_finish(fp);
-    return 1;
-}
-
-/* Dump contents of a list in YAML format */
-static int
-vl_yamldump(vlist *l, FILE *fp)
-{
-    int i;
-
-    VL_CHECK(l);
-
-    if (!v_yaml_start(fp))
-        return 0;
-
-    if (!v_yaml_write_list(l, fp))
-        return 0;
-
-    if (!v_yaml_finish(fp))
-        return 0;
-
-    return 1;
-}
