@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -56,7 +56,6 @@
 #include "vars-memory.h"
 #include "vars-scalar.h"
 #include "vars-system.h"
-#include "vars-yaml.h"
 
 #define FLOAT_FORMAT "%.12g"
 
@@ -77,10 +76,7 @@ struct v_scalar {
 static char buf[BUFSIZ];
 
 /* Type variable */
-vtype *vscalar_type = NULL;
-
-/* Internal functions */
-static int vs_yamldump(vscalar *s, FILE *fp);
+static vtype *vscalar_type = NULL;
 
 /*!
   @brief Return case-independent string comparison of two scalars.
@@ -177,9 +173,7 @@ vs_copy(vscalar *s)
     vc = vs_create(s->type);
     vc->v = s->v;
 
-    if (s->type == V_TYPE_STRING)
-	vc->v.s = V_STRDUP(s->v.s);
-    else if (s->type == V_TYPE_POINTER && v_deepcopy)
+    if (s->type == V_TYPE_POINTER && v_deepcopy)
         vc->v.p = v_copy(s->v.p);
 
     return vc;
@@ -246,8 +240,6 @@ vs_declare(void)
         v_print_func(vscalar_type, vs_print);
         v_freeze_func(vscalar_type, vs_freeze);
         v_thaw_func(vscalar_type, (void *(*)()) vs_thaw);
-        v_xmldump_func(vscalar_type, vs_xmldump);
-        v_yamldump_func(vscalar_type, vs_yamldump);
         v_destroy_func(vscalar_type, vs_destroy);
         v_traverse_func(vscalar_type, vs_traverse);
     }
@@ -273,10 +265,6 @@ void
 vs_destroy(vscalar *s)
 {
     VS_CHECK(s);
-
-    if (s->type == V_TYPE_STRING && !vh_interned(s->v.s))
-	V_DEALLOC(s->v.s);
-
     V_DEALLOC(s);
 }
 
@@ -336,9 +324,6 @@ vs_dstore(vscalar *s, double val)
 
     VS_CHECK(s);
 
-    if (s->type == V_TYPE_STRING)
-	V_DEALLOC(s->v.s);
-
     s->type = V_TYPE_DOUBLE;
     s->v.d = val;
 
@@ -545,9 +530,6 @@ vs_fstore(vscalar *s, float val)
 
     VS_CHECK(s);
 
-    if (s->type == V_TYPE_STRING)
-	V_DEALLOC(s->v.s);
-
     s->type = V_TYPE_FLOAT;
     s->v.f = val;
 
@@ -610,9 +592,6 @@ vs_istore(vscalar *s, int val)
 
     VS_CHECK(s);
 
-    if (s->type == V_TYPE_STRING)
-	V_DEALLOC(s->v.s);
-
     s->type = V_TYPE_INT;
     s->v.i = val;
 
@@ -672,6 +651,7 @@ void
 vs_print(vscalar *s, FILE *fp)
 {
     void *ptr;
+    char *str;
 
     VS_CHECK(s);
 
@@ -693,7 +673,17 @@ vs_print(vscalar *s, FILE *fp)
             v_print(ptr, fp);
         }
     } else {
-        fprintf(fp, "%s\n", vs_sget_buf(s, buf));
+        str = vs_sget_buf(s, buf);
+
+        while (*str != '\0') {
+            if (*str == '\n')
+                fputs("\\n", fp);
+            else
+                fputc(*str, fp);
+            str++;
+        }
+
+        fputc('\n', fp);
     }
 
     v_print_finish();
@@ -716,9 +706,6 @@ vs_pstore(vscalar *s, void *val)
 
     VS_CHECK(s);
 
-    if (s->type == V_TYPE_STRING)
-	V_DEALLOC(s->v.s);
-
     s->type = V_TYPE_POINTER;
     s->v.p = val;
 
@@ -937,11 +924,8 @@ vs_sstore(vscalar *s, char *val)
 
     VS_CHECK(s);
 
-    if (s->type == V_TYPE_STRING)
-	V_DEALLOC(s->v.s);
-
     s->type = V_TYPE_STRING;
-    s->v.s = (val == NULL ? NULL : V_STRDUP(val));
+    s->v.s = vh_intern(val);
 
     return s;
 }
@@ -959,6 +943,7 @@ vs_sstore(vscalar *s, char *val)
 vscalar *
 vs_sstore_len(vscalar *s, char *base, size_t len)
 {
+    char *val;
     int i;
 
     if (s == NULL)
@@ -966,16 +951,18 @@ vs_sstore_len(vscalar *s, char *base, size_t len)
 
     VS_CHECK(s);
 
-    if (s->type == V_TYPE_STRING)
-	V_DEALLOC(s->v.s);
-
-    s->type = V_TYPE_STRING;
-    s->v.s = V_ALLOC(char, len + 1);
+    val = V_ALLOC(char, len + 1);
 
     for (i = 0; i < len; i++)
-        s->v.s[i] = base[i];
+        val[i] = base[i];
+
+    val[len] = '\0';
+
+    s->type = V_TYPE_STRING;
+    s->v.s = vh_intern(val);
+
+    V_DEALLOC(val);
 
-    s->v.s[len] = '\0';
     return s;
 }
 
@@ -1093,10 +1080,6 @@ void
 vs_undef(vscalar *s)
 {
     VS_CHECK(s);
-
-    if (s->type == V_TYPE_STRING)
-	V_DEALLOC(s->v.s);
-
     s->type = V_TYPE_UNDEF;
 }
 
@@ -1136,69 +1119,3 @@ vs_write(vscalar *s, FILE *fp)
 
     return 1;
 }
-
-/* Dump XML scalar to file */
-int
-vs_xmldump(vscalar *s, FILE *fp)
-{
-    void *ptr;
-
-    VS_CHECK(s);
-
-    v_xmldump_start(fp);
-
-    switch(vs_type(s)) {
-
-    case V_TYPE_INT:
-        sprintf(buf, "%d", vs_iget(s));
-        v_xmldump_tag(fp, "int", "value", buf, NULL);
-        break;
-
-    case V_TYPE_FLOAT:
-        sprintf(buf, FLOAT_FORMAT, vs_fget(s));
-        v_xmldump_tag(fp, "float", "value", buf, NULL);
-        break;
-
-    case V_TYPE_DOUBLE:
-        sprintf(buf, FLOAT_FORMAT, vs_dget(s));
-        v_xmldump_tag(fp, "double", "value", buf, NULL);
-        break;
-
-    case V_TYPE_STRING:
-        v_xmldump_data(fp, "string", vs_sget_buf(s, buf));
-        break;
-
-    case V_TYPE_POINTER:
-        ptr = vs_pget(s);
-        if (!v_xmldump(ptr, fp))
-            return 0;
-
-        break;
-
-    case V_TYPE_UNDEF:
-        v_xmldump_tag(fp, "undef", NULL);
-        break;
-    }
-
-    v_xmldump_finish(fp);
-
-    return 1;
-}
-
-/* Dump YAML scalar to file */
-int
-vs_yamldump(vscalar *s, FILE *fp)
-{
-    VS_CHECK(s);
-
-    if (!v_yaml_start(fp))
-        return 0;
-
-    if (!v_yaml_write_scalar(s, fp))
-        return 0;
-
-    if (!v_yaml_finish(fp))
-        return 0;
-
-    return 1;
-}
