@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -78,7 +78,7 @@ void *
 v_read(FILE *fp)
 {
     static vhash *seen = NULL;
-    char *code, idbuf[10];
+    char *code, idbuf[30];
     static int depth = 0;
     void *data = NULL;
     vtype *t;
@@ -107,7 +107,7 @@ v_read(FILE *fp)
         goto end;
 
     if (id > 0) {
-        sprintf(idbuf, "%ld", id);
+        sprintf(idbuf, "%d", id);
         data = vh_pget(seen, idbuf);
         if (data == NULL) {
             /* Read type code */
@@ -657,7 +657,7 @@ v_write(void *ptr, FILE *fp)
     static int count = 0;
     static int depth = 0;
     int id, zero = 0;
-    vtype *t;
+    vtype *t = NULL;
 
     /* Get pointer type */
     if (ptr != NULL) {
