@@ -1,6 +1,6 @@
 /*
  * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
- * Glenn Hutchings 1997-2008.
+ * Glenn Hutchings 1997-2015.
  *
  * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
  * are welcome to redistribute it under certain conditions; see the file
@@ -34,13 +34,10 @@
 #include <vars-iterate.h>
 #include <vars-macros.h>
 #include <vars-memory.h>
-#include <vars-random.h>
 #include <vars-system.h>
 #include <vars-traverse.h>
 #include <vars-type.h>
 #include <vars-write.h>
 #include <vars-utils.h>
-#include <vars-xml.h>
-#include <vars-yaml.h>
 
 #endif
